@@ -206,10 +206,10 @@ swfdec_as_object_hash_create (SwfdecAsObject *object, const char *variable, guin
 {
   SwfdecAsVariable *var;
 
-  if (!swfdec_as_context_use_mem (object->context, sizeof (SwfdecAsVariable)))
-    return NULL;
   if (!swfdec_as_variable_name_is_valid (variable))
     return NULL;
+  if (!swfdec_as_context_use_mem (object->context, sizeof (SwfdecAsVariable)))
+    return NULL;
   var = g_slice_new0 (SwfdecAsVariable);
   var->flags = flags;
   g_hash_table_insert (object->properties, (gpointer) variable, var);
@@ -958,6 +958,7 @@ swfdec_as_object_get_variable_and_flags (SwfdecAsObject *object,
   }
   if (i > SWFDEC_AS_OBJECT_PROTOTYPE_RECURSION_LIMIT) {
     swfdec_as_context_abort (object->context, "Prototype recursion limit exceeded");
+    SWFDEC_AS_VALUE_SET_UNDEFINED (value);
     *flags = 0;
     *pobject = NULL;
     return FALSE;
@@ -1275,13 +1276,14 @@ swfdec_as_object_create (SwfdecAsFunction *fun, guint n_args,
   SwfdecAsContext *context;
   SwfdecAsFunction *cur;
   SwfdecAsFrame *frame;
-  guint size = 0;
+  guint i, size = 0;
   GType type = 0;
 
   g_return_if_fail (SWFDEC_IS_AS_FUNCTION (fun));
 
   context = SWFDEC_AS_OBJECT (fun)->context;
   cur = fun;
+  i = 0;
   do {
     if (SWFDEC_IS_AS_NATIVE_FUNCTION (cur)) {
       SwfdecAsNativeFunction *native = SWFDEC_AS_NATIVE_FUNCTION (cur);
@@ -1291,6 +1293,7 @@ swfdec_as_object_create (SwfdecAsFunction *fun, guint n_args,
 	break;
       }
     }
+    i++;
     swfdec_as_object_get_variable (SWFDEC_AS_OBJECT (cur), SWFDEC_AS_STR_prototype, &val);
     if (SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
       SwfdecAsObject *proto = SWFDEC_AS_VALUE_GET_OBJECT (&val);
@@ -1303,7 +1306,7 @@ swfdec_as_object_create (SwfdecAsFunction *fun, guint n_args,
       }
     }
     cur = NULL;
-  } while (type == 0 && cur != NULL);
+  } while (type == 0 && cur != NULL && i < SWFDEC_AS_OBJECT_PROTOTYPE_RECURSION_LIMIT);
   if (type == 0) {
     type = SWFDEC_TYPE_AS_OBJECT;
     size = sizeof (SwfdecAsObject);
