@@ -1,6 +1,7 @@
 /* This is a generated file. See swfdec_as_strings.c for details. */
 #ifndef _SWFDEC_AS_STRINGS_H_
 #define _SWFDEC_AS_STRINGS_H_
+G_BEGIN_DECLS
 extern const char swfdec_as_strings[];
 #define SWFDEC_AS_STR_EMPTY &swfdec_as_strings[1]
 #define SWFDEC_AS_STR___proto__ &swfdec_as_strings[3]
@@ -284,7 +285,7 @@ extern const char swfdec_as_strings[];
 #define SWFDEC_AS_STR_windowlessDisable &swfdec_as_strings[2969]
 #define SWFDEC_AS_STR_hasTLS &swfdec_as_strings[2988]
 #define SWFDEC_AS_STR_serverString &swfdec_as_strings[2996]
-#define SWFDEC_AS_STR_$version &swfdec_as_strings[3010]
+#define SWFDEC_AS_STR__version &swfdec_as_strings[3010]
 #define SWFDEC_AS_STR_contentType &swfdec_as_strings[3020]
 #define SWFDEC_AS_STR_application_x_www_form_urlencoded &swfdec_as_strings[3033]
 #define SWFDEC_AS_STR_ignoreWhite &swfdec_as_strings[3068]
@@ -470,4 +471,6 @@ extern const char swfdec_as_strings[];
 #define SWFDEC_AS_STR_height &swfdec_as_strings[4903]
 #define SWFDEC_AS_STR_deblocking &swfdec_as_strings[4911]
 #define SWFDEC_AS_STR_smoothing &swfdec_as_strings[4923]
+#define SWFDEC_AS_STR_Invalid_Date &swfdec_as_strings[4934]
+G_END_DECLS
 #endif
